package ru.dbotthepony.kstarbound.defs

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableSet
import ru.dbotthepony.kommons.vector.Vector2d
import ru.dbotthepony.kstarbound.json.builder.IStringSerializable
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.network.syncher.legacyCodec
import ru.dbotthepony.kstarbound.network.syncher.nativeCodec
import java.io.DataInputStream
import java.io.DataOutputStream

enum class TeamType(override val jsonName: String) : IStringSerializable {
	NULL("null"),
	// non-PvP-enabled players and player allied NPCs
	FRIENDLY("friendly"),
	// hostile and neutral NPCs and monsters
	ENEMY("enemy"),
	// PvP-enabled players
	PVP("pvp"),
	// cannot damage anything, can be damaged by Friendly/PVP/Assistant
	PASSIVE("passive"),
	// cannot damage or be damaged
	GHOSTLY("ghostly"),
	// cannot damage enemies, can be damaged by anything except enemy
	ENVIRONMENT("environment"),
	// damages anything except ghostly, damaged by anything except ghostly/passive
	// used for self damage
	INDISCRIMINATE("indiscriminate"),
	// cannot damage friendlies and cannot be damaged by anything
	ASSISTANT("assistant");
}

enum class HitType(override val jsonName: String) : IStringSerializable {
	HIT("Hit"),
	STRONG_HIT("StrongHit"),
	WEAK_HIT("WeakHit"),
	SHIELD_HIT("ShieldHit"),
	KILL("Kill");
}

enum class DamageType(override val jsonName: String) : IStringSerializable {
	NO_DAMAGE("NoDamage"),
	DAMAGE("Damage"),
	IGNORE_DEFENCE("IgnoresDef"),
	KNOCKBACK("IgnoresDef"),
	ENVIRONMENT("Environment"),
	STATUS("Environment");
}

@JsonFactory
data class EntityDamageTeam(val type: TeamType = TeamType.NULL, val team: Int = 0) {
	constructor(stream: DataInputStream, isLegacy: Boolean) : this(TeamType.entries[stream.readUnsignedByte()], stream.readUnsignedShort())

	fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.writeByte(type.ordinal)
		stream.writeShort(team)
	}

	companion object {
		val CODEC = nativeCodec(::EntityDamageTeam, EntityDamageTeam::write)
		val LEGACY_CODEC = legacyCodec(::EntityDamageTeam, EntityDamageTeam::write)
	}
}

@JsonFactory
data class TouchDamage(
	val poly: ImmutableList<Vector2d> = ImmutableList.of(),
	val teamType: TeamType = TeamType.ENVIRONMENT,
	val damage: Double = 0.0,
	val damageSourceKind: String = "",
	val knockback: Double = 0.0,
	val statusEffects: ImmutableSet<String> = ImmutableSet.of(),
)
