package ru.dbotthepony.kstarbound.defs.world

import ru.dbotthepony.kommons.math.RGBAColor
import ru.dbotthepony.kommons.vector.Vector2d
import ru.dbotthepony.kstarbound.GlobalDefaults
import ru.dbotthepony.kstarbound.Registries
import ru.dbotthepony.kstarbound.util.random.random
import kotlin.properties.Delegates

class FloatingDungeonWorldParameters : VisitableWorldParameters() {
	var dungeonBaseHeight: Int by Delegates.notNull()
		private set
	var dungeonSurfaceHeight: Int by Delegates.notNull()
		private set
	var dungeonUndergroundLevel: Int by Delegates.notNull()
		private set
	var primaryDungeon: String by Delegates.notNull()
		private set
	var ambientLightLevel: RGBAColor by Delegates.notNull()
		private set
	var biome: String? = null
		private set
	var dayMusicTrack: String? = null
		private set
	var nightMusicTrack: String? = null
		private set
	var dayAmbientNoises: String? = null
		private set
	var nightAmbientNoises: String? = null
		private set

	override val type: VisitableWorldParametersType
		get() = VisitableWorldParametersType.FLOATING_DUNGEON

	override fun createLayout(seed: Long): WorldLayout {
		val random = random(seed)

		val layout = WorldLayout()
		layout.worldSize = worldSize

		layout.addLayer(random, 0, WorldLayout.RegionParameters(
			dungeonSurfaceHeight, threatLevel, biome,
			null, null, null, null, null, null,
			WorldLayout.RegionLiquids()
		))

		val color = if (biome != null) {
			// TODO: Original game engine queries biome database but forgets to finalize world layout if biome is present
			// TODO: Is that intentional?
			// (: god damn it.

			Registries.biomes[biome!!]?.value?.skyColoring(random)?.mainColor ?: RGBAColor.BLACK
		} else {
			RGBAColor.BLACK
		}

		layout.finalize(color)
		return layout
	}

	companion object {
		fun generate(typeName: String): FloatingDungeonWorldParameters {
			val config = GlobalDefaults.dungeonWorlds[typeName] ?: throw NoSuchElementException("Unknown dungeon world type $typeName!")
			val parameters = FloatingDungeonWorldParameters()

			parameters.threatLevel = config.threatLevel
			parameters.gravity = config.gravity.map({ Vector2d(y = it) }, { it })
			parameters.airless = config.airless
			parameters.environmentStatusEffects = config.environmentStatusEffects

			parameters.overrideTech = config.overrideTech
			parameters.globalDirectives = config.globalDirectives
			parameters.beamUpRule = config.beamUpRule
			parameters.disableDeathDrops = config.disableDeathDrops
			parameters.worldEdgeForceRegions = config.worldEdgeForceRegions
			parameters.weatherPool = config.weatherPool
			parameters.dungeonBaseHeight = config.dungeonBaseHeight
			parameters.dungeonSurfaceHeight = config.dungeonSurfaceHeight
			parameters.dungeonUndergroundLevel = config.dungeonUndergroundLevel
			parameters.primaryDungeon = config.primaryDungeon
			parameters.biome = config.biome
			parameters.ambientLightLevel = config.ambientLightLevel

			if (config.musicTrack != null) {
				parameters.dayMusicTrack = config.musicTrack.fullPath
				parameters.nightMusicTrack = config.musicTrack.fullPath
			} else {
				parameters.dayMusicTrack = config.dayMusicTrack?.fullPath
				parameters.nightMusicTrack = config.nightMusicTrack?.fullPath
			}

			if (config.ambientNoises != null) {
				parameters.dayAmbientNoises = config.ambientNoises.fullPath
				parameters.nightAmbientNoises = config.ambientNoises.fullPath
			} else {
				parameters.dayAmbientNoises = config.dayAmbientNoises?.fullPath
				parameters.nightAmbientNoises = config.nightAmbientNoises?.fullPath
			}

			return parameters
		}
	}
}
