package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.kommons.util.setValue
import ru.dbotthepony.kommons.util.value
import ru.dbotthepony.kstarbound.defs.world.SkyParameters
import ru.dbotthepony.kstarbound.defs.world.SkyType
import ru.dbotthepony.kstarbound.defs.world.WarpPhase
import ru.dbotthepony.kstarbound.network.syncher.NetworkedGroup
import ru.dbotthepony.kstarbound.network.syncher.MasterElement
import ru.dbotthepony.kstarbound.network.syncher.networkedBoolean
import ru.dbotthepony.kstarbound.network.syncher.networkedDouble
import ru.dbotthepony.kstarbound.network.syncher.networkedEnumStupid
import ru.dbotthepony.kstarbound.network.syncher.networkedFloat
import ru.dbotthepony.kstarbound.network.syncher.networkedJson
import ru.dbotthepony.kstarbound.network.syncher.networkedUnsignedInt
import ru.dbotthepony.kstarbound.network.syncher.networkedVec2f

class Sky() {
	private val skyParametersNetState = networkedJson(SkyParameters())

	private val skyTypeNetState = networkedEnumStupid(SkyType.ORBITAL)
	private val timeNetState = networkedDouble()
	private val flyingTypeNetState = networkedUnsignedInt()
	private val enterHyperspaceNetState = networkedBoolean()
	private val startInWarpNetState = networkedBoolean()
	private val worldMoveNetState = networkedEnumStupid(WarpPhase.MAINTAIN)
	private val starMoveNetState = networkedVec2f()
	private val warpPhaseNetState = networkedVec2f()
	private val flyingTimerNetState = networkedFloat()

	var skyType by skyTypeNetState
		private set
	var time by timeNetState
		private set
	var flyingType by flyingTypeNetState
		private set
	var enterHyperspace by enterHyperspaceNetState
		private set
	var startInWarp by startInWarpNetState
		private set
	var worldMove by worldMoveNetState
		private set
	var starMove by starMoveNetState
		private set
	var warpPhase by warpPhaseNetState
		private set
	var flyingTimer by flyingTimerNetState
		private set

	val networkedGroup = MasterElement(NetworkedGroup(
		skyParametersNetState,
		skyTypeNetState,
		timeNetState,
		flyingTypeNetState,
		enterHyperspaceNetState,
		startInWarpNetState,
		worldMoveNetState,
		starMoveNetState,
		warpPhaseNetState,
		flyingTimerNetState,
	))

	constructor(parameters: SkyParameters, inOrbit: Boolean) : this() {
		skyParametersNetState.value = parameters.copy()

		if (inOrbit) {
			skyType = SkyType.ORBITAL
		} else {
			skyType = parameters.skyType
		}
	}
}
