package ru.dbotthepony.kstarbound.world.entities

import ru.dbotthepony.kommons.math.RGBAColor
import ru.dbotthepony.kommons.util.AABB
import ru.dbotthepony.kommons.vector.Vector2d
import ru.dbotthepony.kstarbound.client.StarboundClient
import ru.dbotthepony.kstarbound.client.render.LayeredRenderer
import ru.dbotthepony.kstarbound.client.render.RenderLayer
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.World
import java.util.function.Consumer

/**
 * Entities with dynamics (Player, Drops, Projectiles, NPCs, etc)
 */
abstract class DynamicEntity(path: String) : AbstractEntity(path) {
	private var forceChunkRepos = false

	override var position
		get() = movement.position
		set(value) {
			movement.position = value
		}

	abstract val movement: MovementController

	final override var chunkPos: ChunkPos = ChunkPos.ZERO
		private set

	override fun onJoinWorld(world: World<*, *>) {
		world.dynamicEntities.add(this)
		movement.world = world
		forceChunkRepos = true

		/*movement.positionListeners.addListener(Consumer { field ->
			val oldChunkPos = world.geometry.chunkFromCell(old)
			val newChunkPos = world.geometry.chunkFromCell(field)

			chunkPos = newChunkPos

			if (oldChunkPos != newChunkPos || forceChunkRepos) {
				chunk = world.chunkMap[newChunkPos]
				forceChunkRepos = false
			}
		})*/
	}

	override fun onRemove(world: World<*, *>) {
		world.dynamicEntities.remove(this)
	}

	override fun render(client: StarboundClient, layers: LayeredRenderer) {
		layers.add(RenderLayer.Overlay.point()) {
			val hitboxes = movement.localHitboxes.toList()
			if (hitboxes.isEmpty()) return@add

			hitboxes.forEach { it.render(client) }

			world.queryCollisions(
				hitboxes.stream().map { it.aabb }.reduce(AABB::combine).get().enlarge(2.0, 2.0)
			).filter(movement::shouldCollideWithBody).forEach { it.poly.render(client, BLOCK_COLLISION_COLOR) }
		}
	}

	companion object {
		val BLOCK_COLLISION_COLOR = RGBAColor(65, 179, 217)
	}
}
